<?php

namespace Drupal\campaign_kit\Controller;

use Drupal\campaign_kit_core\Entity\Campaign;
use Drupal\campaign_kit_core\Entity\CampaignDonation;
use Drupal\campaign_kit_core\Entity\Team;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityFormBuilderInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Link;
use Drupal\Core\Path\AliasManager;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Routing\RouteProvider;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Class CampaignKitController.
 */
class CampaignKitController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity form builder.
   *
   * @var \Drupal\Core\Entity\EntityFormBuilderInterface
   */
  protected $entityFormBuilder;

  /**
   * The route provider.
   *
   * @var \Drupal\Core\Routing\RouteProvider
   */
  protected $routeProvider;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The path alias manager.
   *
   * @var \Drupal\Core\Path\AliasManager
   */
  protected $pathAliasManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * The account proxy interface.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The current path stack.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * The currently active request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The current active database's master connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The service date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * CampaignKitController constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityFormBuilderInterface $entityFormBuilder
   *   The entity form builder.
   * @param \Drupal\Core\Routing\RouteProvider $routeProvider
   *   The route provider.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   * @param \Drupal\Core\Path\AliasManager $pathAliasManager
   *   The path alias manager.
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   The config factory.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   The account proxy interface.
   * @param \Drupal\Core\Path\CurrentPathStack $currentPath
   *   The current path stack.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The currently active request object.
   * @param \Drupal\Core\Database\Connection $connection
   *   The current active database's master connection.
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   The service date formatter.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, EntityFormBuilderInterface $entityFormBuilder, RouteProvider $routeProvider, ModuleHandlerInterface $moduleHandler, AliasManager $pathAliasManager, ConfigFactory $configFactory, AccountProxyInterface $currentUser, CurrentPathStack $currentPath, Request $request, Connection $connection, DateFormatter $dateFormatter) {
    $this->entityTypeManager = $entityTypeManager;
    $this->entityFormBuilder = $entityFormBuilder;
    $this->routeProvider = $routeProvider;
    $this->moduleHandler = $moduleHandler;
    $this->pathAliasManager = $pathAliasManager;
    $this->configFactory = $configFactory;
    $this->currentUser = $currentUser;
    $this->currentPath = $currentPath;
    $this->request = $request;
    $this->connection = $connection;
    $this->dateFormatter = $dateFormatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /* @var \Symfony\Component\DependencyInjection\ContainerInterface */
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity.form_builder'),
      $container->get('router.route_provider'),
      $container->get('module_handler'),
      $container->get('path.alias_manager'),
      $container->get('config.factory'),
      $container->get('current_user'),
      $container->get('path.current'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('database'),
      $container->get('date.formatter')
    );
  }

  /**
   * Function to show Campaign links.
   *
   * @return mixed
   *   Returns links.
   */
  public function campaignKitPage() {
    $links = [];

    // Campaign list.
    if (count($this->routeProvider->getRoutesByNames(['entity.campaign.collection'])) === 1) {
      $campaignList = Url::fromRoute('entity.campaign.collection');
      $links[] = Link::fromTextAndUrl($this->t('Campaigns'), $campaignList);
    }

    // Campaign Donation list.
    if (count($this->routeProvider->getRoutesByNames(['entity.campaign_donation.collection'])) === 1) {
      $campaignDonationList = Url::fromRoute('entity.campaign_donation.collection');
      $links[] = Link::fromTextAndUrl($this->t('Campaign donations'), $campaignDonationList);
    }

    // Campaign Update list.
    if (count($this->routeProvider->getRoutesByNames(['entity.campaign_update.collection'])) === 1) {
      $campaignUpdateList = Url::fromRoute('entity.campaign_update.collection');
      $links[] = Link::fromTextAndUrl($this->t('Campaign updates'), $campaignUpdateList);
    }

    // Campaign Transaction list.
    if (count($this->routeProvider->getRoutesByNames(['entity.campaign_transaction.collection'])) === 1) {
      $campaignTransactionList = Url::fromRoute('entity.campaign_transaction.collection');
      $links[] = Link::fromTextAndUrl($this->t('Campaign transactions'), $campaignTransactionList);
    }

    // Team list.
    if (count($this->routeProvider->getRoutesByNames(['entity.team.collection'])) === 1) {
      $teamList = Url::fromRoute('entity.team.collection');
      $links[] = Link::fromTextAndUrl($this->t('Teams'), $teamList);
    }

    // Donation list.
    if (count($this->routeProvider->getRoutesByNames(['entity.donation.collection'])) === 1) {
      $donationList = Url::fromRoute('entity.donation.collection');
      $links[] = Link::fromTextAndUrl($this->t('Donations (not ready)'), $donationList);
    }

    $rows = [];
    foreach ($links as $link) {
      $rows[] = [
        '#markup' => $link,
      ];
    }

    $response['campaign_kit_table'] = [
      '#rows' => $rows,
      '#header' => [$this->t('Campaign Kit')],
      '#type' => 'table',
      '#empty' => $this->t('Please enable Campaign Kit Core module to see the menu.'),
    ];

    $links = [];

    // Campaign Types.
    if (count($this->routeProvider->getRoutesByNames(['entity.campaign_type.collection'])) === 1) {
      $campaignTypesForm = Url::fromRoute('entity.campaign_type.collection');
      $links[] = Link::fromTextAndUrl($this->t('Campaign Types'), $campaignTypesForm);
    }

    // Team Types.
    if (count($this->routeProvider->getRoutesByNames(['entity.team_type.collection'])) === 1) {
      $teamTypesForm = Url::fromRoute('entity.team_type.collection');
      $links[] = Link::fromTextAndUrl($this->t('Team Types'), $teamTypesForm);
    }

    // Campaign Kit Configuration Form.
    if (count($this->routeProvider->getRoutesByNames(['campaign_kit.configuration_form'])) === 1) {
      $campaignKitConfigForm = Url::fromRoute('campaign_kit.configuration_form');
      $links[] = Link::fromTextAndUrl($this->t('Campaign Kit Configuration Form'), $campaignKitConfigForm);
    }

    $rows = [];
    foreach ($links as $link) {
      $rows[] = [
        '#markup' => $link,
      ];
    }

    $response['campaign_kit_config_table'] = [
      '#rows' => $rows,
      '#header' => [$this->t('Campaign Kit &mdash; Configuration')],
      '#type' => 'table',
      '#empty' => $this->t('No content available.'),
    ];

    // Add links only if demo module is installed.
    $rows = [];
    if ($this->moduleHandler->moduleExists('campaign_kit_demo')) {
      // Query demo Campaigns created by Campaign Kit Demo module.
      $ids = $this->entityTypeManager->getStorage('campaign')->getQuery('AND')
        ->condition('user_id', '0', '=')
        ->range('0', '3')
        ->execute();

      /* @var $entries \Drupal\campaign_kit_core\Entity\Campaign */
      $entries = Campaign::loadMultiple($ids);
      foreach ($entries as $entry => $value) {
        $campaignUrl = Url::fromRoute('campaign_kit.campaign_kit_controller_view_campaign_entity', ['campaign' => $value->id()]);
        $campaignLink = Link::fromTextAndUrl($value->label(), $campaignUrl);
        $rows[] = [
          '#markup' => $campaignLink,
        ];
      }

      // Get Badge Page link.
      // TODO: Revisit badges.
      $badgePage = Url::fromRoute('campaign_kit.campaign_kit_controller_badge_page');
      $linkBadgePage = Link::fromTextAndUrl($this->t('Badges'), $badgePage);

      $badges = [
        $linkBadgePage,
      ];

      foreach ($badges as $badge) {
        $rows[] = [
          '#markup' => $badge,
        ];
      }
    }

    $response['campaign_kit_demo_table'] = [
      '#rows' => $rows,
      '#header' => [$this->t('Campaign Kit &mdash;  Demonstration')],
      '#type' => 'table',
      '#empty' => $rows ? $this->t('No content available.') : $this->t('Please enable Campaign Kit Demonstration module to see examples.'),
    ];

    return $response;
  }

  /**
   * View Campaign entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $campaign
   *   The Campaign entity.
   *
   * @return array
   *   Returns an array.
   */
  public function viewCampaign(EntityInterface $campaign = NULL) {
    // Local variables.
    $campaignArray = [];
    $campaignId = $campaign->id();
    // Check if the entity exists.
    $ids = $this->entityTypeManager->getStorage('campaign')->getQuery('AND')
      ->condition('id', $campaignId, '=')
      ->execute();

    if (is_numeric($campaignId)) {
      /*
       * @var $campaignEntity \Drupal\campaign_kit_core\Entity\Campaign
       * @var $parent \Drupal\campaign_kit_core\Entity\Campaign
       */
      $campaignEntity = Campaign::load($campaignId);
      if ($campaignEntity->getCampaignType() == 'child') {
        $parent = $campaignEntity->getParent();
        $parentId = $parent->id();
        // Send this campaign to another controller.
        $campaignArray = $this->viewCampaignChild($parentId, $campaign);
      }
      else {
        if ($ids) {
          // View builder.
          $entityType = 'campaign';
          $viewDisplay = 'campaign_kit';

          // Fields to render.
          $fields = [
            'title',
            'parent_id',
            'campaign_type',
            'header_image',
            'confirmation_email',
            'description',
            'short_description',
          ];

          // Get the Campaign entity object.
          /* @var $campaignEntity \Drupal\campaign_kit_core\Entity\Campaign */
          $campaignEntity = Campaign::load($campaignId);
          // Get the View for this Campaign entity object.
          $viewBuilder = $this->entityTypeManager->getViewBuilder($entityType);

          // Build the Campaign array to render.
          foreach ($fields as $field_name) {
            if ($campaignEntity->hasField($field_name) && $campaignEntity->access('view')) {
              $value = $campaignEntity->get($field_name);
              $output = $viewBuilder->viewField($value, $viewDisplay);
              $output['#cache']['tags'] = $campaignEntity->getCacheTags();
              $campaignArray[$field_name] = $output;
            }
          }
          // Get the campaign entity.
          // Build the Campaign object.
          $campaignArray['id'] = $campaignEntity->getCampaignId();
          $campaignArray['one_time_donation_goal'] = (int) $campaignEntity->getGoal();
          $campaignArray['monthly_donation_goal'] = (int) $campaignEntity->getGoalSustaining();
          $campaignArray['campaign_status'] = $campaignEntity->getCampaignStatus();
          $campaignArray['facebook_url'] = $campaignEntity->getFacebookPath();
          $campaignArray['twitter_url'] = $campaignEntity->getTwitterPath();
          $campaignArray['google_url'] = $campaignEntity->getGooglePath();
          $campaignArray['donation_frequency_allowed'] = $campaignEntity->getFrequencyAllowed();
          $campaignArray['campaign_type'] = $campaignEntity->getCampaignType();
          $campaignArray['days_left'] = $campaignEntity->getDaysLeft();
          $campaignArray['goal'] = $campaignEntity->getGoal();
          $campaignArray['author_name'] = $campaignEntity->getOwnerName();
          $campaignArray['use_teams'] = $campaignEntity->get('use_teams')->value;
          // Get alias.
          $aliasCampaign = $this->pathAliasManager->getAliasByPath('/campaign/' . $campaignId);
          if ($aliasCampaign !== '') {
            $campaignArray['alias'] = $aliasCampaign;
          }

          // Get Campaign's money information.
          $campaignArray['num_supporters'] = $campaignEntity->getNumberSupporters();
          $campaignArray['goal_saved'] = $campaignEntity->getRaisedAmount();
          $campaignArray['percentage'] = $campaignEntity->getPercentage();
          // Get currency from plugin.
          $config = $this->configFactory->get('campaign_kit.settings');
          $clientPlugin = $config->get('campaign_kit');
          $campaignArray['currency'] = $clientPlugin['currency'];

          // Create links for Campaign Page.
          // I want to participate link destination.
          $destination = Url::fromRoute('campaign_kit.campaign_kit_controller_add_child_page', ['campaignParentId' => $campaignArray['id']]);

          $loginUrl = Url::fromRoute(
            'user.login',
            [],
            [
              'query' => ['destination' => $destination->getInternalPath()],
            ]
          );

          if ($this->currentUser->isAnonymous()) {
            $linkParticipate = Link::fromTextAndUrl($this->t('I Want to Join this Campaign!'), $loginUrl);
          }
          else {
            $linkParticipate = Link::fromTextAndUrl($this->t('I Want to Join this Campaign!'), $destination);
          }

          $campaignArray['participate_link'] = $linkParticipate->toRenderable();
          $campaignArray['participate_link']['#attributes'] = ['class' => ['donate-button']];

          // Donate link.
          $donationUrl = Url::fromRoute('campaign_kit.donate_form_campaign', ['campaignId' => $campaignArray['id']]);
          $donationLink = Link::fromTextAndUrl($this->t('Donate Now'), $donationUrl);
          $campaignArray['donate_link'] = $donationLink->toRenderable();
          $campaignArray['donate_link']['#attributes'] = ['class' => ['donate-button']];

          // Create social network links.
          if ($campaignEntity->getFacebookPath()) {
            $facebookUrl = Url::fromUri($campaignEntity->getFacebookPath());
            $facebookLink = Link::fromTextAndUrl($this->t('Share on Facebook'), $facebookUrl);
            $campaignArray['facebook_link'] = $facebookLink->toRenderable();
            $campaignArray['facebook_link']['#attributes'] = [
              'class' => ['facebook-icon'],
              'target' => '_blank',
              'title' => $this->t('Share on Facebook'),
            ];
          }

          if ($campaignEntity->getTwitterPath()) {
            $twitterUrl = Url::fromUri($campaignEntity->getTwitterPath());
            $twitterLink = Link::fromTextAndUrl($this->t('Share on Twitter'), $twitterUrl);
            $campaignArray['twitter_link'] = $twitterLink->toRenderable();
            $campaignArray['twitter_link']['#attributes'] = [
              'class' => ['twitter-icon'],
              'target' => '_blank',
              'title' => $this->t('Share on Twitter'),
            ];
          }

          if ($campaignEntity->getGooglePath()) {
            $googleUrl = Url::fromUri($campaignEntity->getGooglePath());
            $googleLink = Link::fromTextAndUrl($this->t('Share on Google+'), $googleUrl);
            $campaignArray['google_link'] = $googleLink->toRenderable();
            $campaignArray['google_link']['#attributes'] = [
              'class' => ['google-icon'],
              'target' => '_blank',
              'title' => $this->t('Share on Google +'),
            ];
          }

          // Get the alias for the current campaign.
          $isItEnabled = $this->moduleHandler->moduleExists('pathauto');
          $routName = $this->currentPath->getPath();
          if ($isItEnabled) {
            $result = $this->pathAliasManager->getAliasByPath($routName);
            if ($result) {
              $bodyLink = $result;
            }
            else {
              $bodyLink = $routName;
            }
          }
          else {
            $bodyLink = $routName;
          }
          $hostHttp = $this->request->getSchemeAndHttpHost();
          $bodyLink = $hostHttp . $bodyLink;
          // Current URL.
          $campaignArray['current_url'] = $bodyLink;

          // Create link for email a campaign.
          $subject = $this->t('Check out this fantastic campaign!');
          $body = $this->t('Have a look at this campaign and please help if you are inspired to!');
          $friendUrl = Url::fromUri('mailto:?subject=' . $subject . '&body=' . $body . '%0D%0A' . $bodyLink);
          $friendLink = Link::fromTextAndUrl($this->t('Email a friend'), $friendUrl);
          $campaignArray['email_friend_link'] = $friendLink->toRenderable();
          $campaignArray['email_friend_link']['#attributes'] = [
            'class' => ['email-icon'],
            'title' => $this->t('Email a friend'),
          ];

          // Create Subscribe link.
          $subscribeUrl = Url::fromUri('https://www.facebook.com/');
          $subscribeLink = Link::fromTextAndUrl($this->t('Subscribe to Updates'), $subscribeUrl);
          $campaignArray['subscribe_link'] = $subscribeLink->toRenderable();
          $campaignArray['subscribe_link']['#attributes'] = [
            'class' => ['subscribe-button'],
            'title' => $this->t('Subscribe to Updates'),
          ];

          // Create Continue to homepage link.
          $homeUrl = Url::fromRoute('<front>');
          $homeLink = Link::fromTextAndUrl($this->t('Continue to homepage'), $homeUrl);
          $campaignArray['home_page_link'] = $homeLink->toRenderable();
          $campaignArray['home_page_link']['#attributes'] = [
            'class' => ['default-button'],
            'title' => $this->t('Continue to homepage'),
          ];

          // Get views and add them to Campaign array.
          $renderSupporters = views_embed_view('campaign_kit_supporters', [
            'display_id' => 'block_1',
            'check_access' => TRUE,
          ]);
          $renderCampaignUpdates = views_embed_view('campaign_kit_updates', [
            'display_id' => 'block_1',
            'check_access' => TRUE,
          ]);

          $campaignArray['supporters_view'] = $renderSupporters;
          $campaignArray['updates_view'] = $renderCampaignUpdates;

        }
        else {
          drupal_set_message($this->t("Unable to find campaign."), 'error');
        }
      }
    }

    $response = [
      '#theme' => 'campaign--display',
      '#campaign' => $campaignArray,
    ];

    return $response;
  }

  /**
   * View Campaign Child entity.
   *
   * @param int $campaignParentId
   *   The parent id.
   * @param \Drupal\Core\Entity\EntityInterface $campaign
   *   The Campaign entity.
   *
   * @return array
   *   Returns an array.
   */
  public function viewCampaignChild($campaignParentId, EntityInterface $campaign) {
    // Local variables.
    $campaignArray = [];
    $campaignChildId = $campaign->id();

    // Check if the Campaign parent entity exists.
    $parentId = $this->entityTypeManager->getStorage('campaign')->getQuery('AND')
      ->condition('id', $campaignParentId, '=')
      ->condition('campaign_type', 'parent', '=')
      ->execute();
    if ($parentId) {
      // Check if the Campaign child entity exists.
      $childId = $this->entityTypeManager->getStorage('campaign')->getQuery('AND')
        ->condition('id', $campaignChildId, '=')
        ->condition('campaign_type', 'child', '=')
        ->execute();

      if ($childId) {
        // View builder.
        $entityType = 'campaign';
        $viewDisplay = 'campaign_kit';

        // Fields to render.
        $fields = [
          'title',
          'parent_id',
          'campaign_type',
          'header_image',
          'confirmation_email',
          'description',
          'short_description',
        ];

        // Get the Campaign entity object.
        /* @var $campaignEntity \Drupal\campaign_kit_core\Entity\Campaign */
        $campaignEntity = Campaign::load($campaignChildId);
        // Get the View for this Campaign entity object.
        $viewBuilder = $this->entityTypeManager->getViewBuilder($entityType);

        // Build the Campaign array to render.
        foreach ($fields as $field_name) {
          if ($campaignEntity->hasField($field_name) && $campaignEntity->access('view')) {
            $value = $campaignEntity->get($field_name);
            $output = $viewBuilder->viewField($value, $viewDisplay);
            $output['#cache']['tags'] = $campaignEntity->getCacheTags();
            $campaignArray[$field_name] = $output;
          }
        }
        // Get the campaign entity.
        // Build the Campaign object.
        $campaignArray['id'] = $campaignEntity->getCampaignId();
        $campaignArray['one_time_donation_goal'] = (int) $campaignEntity->getGoal();
        $campaignArray['monthly_donation_goal'] = (int) $campaignEntity->getGoalSustaining();
        $campaignArray['campaign_status'] = $campaignEntity->getCampaignStatus();
        $campaignArray['facebook_url'] = $campaignEntity->getFacebookPath();
        $campaignArray['twitter_url'] = $campaignEntity->getTwitterPath();
        $campaignArray['google_url'] = $campaignEntity->getGooglePath();
        $campaignArray['donation_frequency_allowed'] = $campaignEntity->getFrequencyAllowed();
        $campaignArray['campaign_type'] = $campaignEntity->getCampaignType();
        $campaignArray['days_left'] = $campaignEntity->getDaysLeft();
        $campaignArray['goal'] = (int) $campaignEntity->getGoal();
        $campaignArray['author_name'] = $campaignEntity->getOwnerName();

        // Get Campaign's money information.
        $campaignArray['num_supporters'] = $campaignEntity->getNumberSupporters();
        $campaignArray['goal_saved'] = $campaignEntity->getRaisedAmount();
        $campaignArray['percentage'] = $campaignEntity->getPercentage();
        // Get currency from plugin.
        $config = $this->configFactory->get('campaign_kit.settings');
        $clientPlugin = $config->get('campaign_kit');
        $campaignArray['currency'] = $clientPlugin['currency'];

        // Get alias by path.
        $campaignChildAlias = $this->pathAliasManager->getAliasByPath('/campaign/' . $campaignArray['id']);
        $campaignChildAlias = str_replace('/campaign/', '', $campaignChildAlias);

        // Create links for Campaign Page.
        // Donate link.
        $donationUrl = Url::fromRoute('campaign_kit.donate_form_campaign_child', [
          'campaignId' => $campaignParentId,
          'campaignChildId' => $campaignChildAlias,
        ]);
        $donationLink = Link::fromTextAndUrl($this->t('Donate Now'), $donationUrl);
        $campaignArray['donate_link'] = $donationLink->toRenderable();
        $campaignArray['donate_link']['#attributes'] = ['class' => ['donate-button']];

        // Create social network links.
        if ($campaignEntity->getFacebookPath()) {
          $facebookUrl = Url::fromUri($campaignEntity->getFacebookPath());
          $facebookLink = Link::fromTextAndUrl($this->t('Share on Facebook'), $facebookUrl);
          $campaignArray['facebook_link'] = $facebookLink->toRenderable();
          $campaignArray['facebook_link']['#attributes'] = [
            'class' => ['facebook-icon'],
            'target' => '_blank',
            'title' => $this->t('Share on Facebook'),
          ];
        }

        if ($campaignEntity->getTwitterPath()) {
          $twitterUrl = Url::fromUri($campaignEntity->getTwitterPath());
          $twitterLink = Link::fromTextAndUrl($this->t('Share on Twitter'), $twitterUrl);
          $campaignArray['twitter_link'] = $twitterLink->toRenderable();
          $campaignArray['twitter_link']['#attributes'] = [
            'class' => ['twitter-icon'],
            'target' => '_blank',
            'title' => $this->t('Share on Twitter'),
          ];
        }

        if ($campaignEntity->getGooglePath()) {
          $googleUrl = Url::fromUri($campaignEntity->getGooglePath());
          $googleLink = Link::fromTextAndUrl($this->t('Share on Google+'), $googleUrl);
          $campaignArray['google_link'] = $googleLink->toRenderable();
          $campaignArray['google_link']['#attributes'] = [
            'class' => ['google-icon'],
            'target' => '_blank',
            'title' => $this->t('Share on Google +'),
          ];
        }

        // Get the alias for the current campaign.
        $isItEnabled = $this->moduleHandler->moduleExists('pathauto');
        $routName = $this->currentPath->getPath();
        if ($isItEnabled) {
          $result = $this->pathAliasManager->getAliasByPath($routName);
          if ($result) {
            $bodyLink = $result;
          }
          else {
            $bodyLink = $routName;
          }
        }
        else {
          $bodyLink = $routName;
        }
        $hostHttp = $this->request->getSchemeAndHttpHost();
        $bodyLink = $hostHttp . $bodyLink;
        // Current URL.
        $campaignArray['current_url'] = $bodyLink;

        // Create link for email a campaign.
        $subject = $this->t('Check out this fantastic campaign!');
        $body = $this->t('Have a look at this campaign and please help if you are inspired to!');
        $friendUrl = Url::fromUri('mailto:?subject=' . $subject . '&body=' . $body . '%0D%0A' . $bodyLink);
        $friendLink = Link::fromTextAndUrl($this->t('Email a friend'), $friendUrl);
        $campaignArray['email_friend_link'] = $friendLink->toRenderable();
        $campaignArray['email_friend_link']['#attributes'] = [
          'class' => ['email-icon'],
          'title' => $this->t('Email a friend'),
        ];

        // Create Subscribe link.
        $subscribeUrl = Url::fromUri('https://www.facebook.com/');
        $subscribeLink = Link::fromTextAndUrl($this->t('Subscribe to Updates'), $subscribeUrl);
        $campaignArray['subscribe_link'] = $subscribeLink->toRenderable();
        $campaignArray['subscribe_link']['#attributes'] = [
          'class' => ['subscribe-button'],
          'title' => $this->t('Subscribe to Updates'),
        ];

        // Create Continue to homepage link.
        $homeUrl = Url::fromRoute('<front>');
        $homeLink = Link::fromTextAndUrl($this->t('Continue to homepage'), $homeUrl);
        $campaignArray['home_page_link'] = $homeLink->toRenderable();
        $campaignArray['home_page_link']['#attributes'] = [
          'class' => ['subscribe-icon'],
          'title' => $this->t('Continue to homepage'),
        ];

        // Render Supporters and Campaign Update views.
        $renderSupporters = views_embed_view('campaign_kit_supporters', [
          'display_id' => 'block_1',
          'check_access' => TRUE,
        ]);
        $renderCampaignUpdates = views_embed_view('campaign_kit_updates', [
          'display_id' => 'block_1',
          'check_access' => TRUE,
        ]);
        $campaignArray['supporters_view'] = $renderSupporters;
        $campaignArray['updates_view'] = $renderCampaignUpdates;

      }
      else {
        drupal_set_message($this->t("Unable to find campaign."), 'error');
      }
    }
    else {
      drupal_set_message($this->t("Unable to find campaign."), 'error');
    }

    return $campaignArray;
  }

  /**
   * Team entity view.
   *
   * @param int $teamId
   *   The team id.
   *
   * @return array
   *   Return Template for Badge page.
   */
  public function teamView($teamId) {
    // Check if the Team entity exists.
    $teamId = $this->entityTypeManager->getStorage('team')->getQuery('AND')
      ->condition('id', $teamId, '=')
      ->execute();

    if ($teamId) {
      // Get the Team entity.
      /* @var $teamEntity \Drupal\campaign_kit_core\Entity\Team */
      $teamEntity = Team::load(array_shift($teamId));

      $teamArray = [];
      $teamArray['name'] = $teamEntity->getName();
      $teamArray['available'] = $teamEntity->getAvailable();
      $teamArray['is_published'] = $teamEntity->isPublished();

      $response = [
        '#theme' => 'team--display',
        '#team' => $teamArray,
      ];
    }
    else {
      $response = [
        '#markup' => 'This team does not exist.',
      ];

    }
    return $response;
  }

  /**
   * Badge page.
   *
   * @return array
   *   Return Template for Badge page.
   */
  public function badge() {
    // Let's check if our demo module is enabled.
    $isItEnabled = $this->moduleHandler->moduleExists('campaign_kit_demo');

    $renderBadge = views_embed_view('campaign_kit_badges', [
      'display_id' => 'block_1',
      'check_access' => TRUE,
    ]);

    $response = [
      '#theme' => 'campaign-badge',
      '#enable' => $isItEnabled,
      '#badge' => $renderBadge,
    ];

    return $response;
  }

  /**
   * Create Child Campaign form.
   *
   * @return array
   *   Returns create child form.
   */
  public function addChild() {
    $campaign = $this->entityTypeManager->getStorage('campaign')
      ->create(['type' => 'child']);
    $childCreateForm = $this->entityFormBuilder->getForm($campaign, 'child');

    $response = [
      '#theme' => 'campaign--child--add',
      '#child_create_form' => $childCreateForm,
    ];
    return $response;
  }

  /**
   * Thank you with Campaign ID page.
   *
   * @param \Drupal\Core\Entity\EntityInterface $campaign
   *   The Campaign entity.
   * @param int $campaignDonationId
   *   The Campaign Donation ID.
   *
   * @return array
   *   Returns an array for Thank You page.
   */
  public function thankYouId(EntityInterface $campaign, $campaignDonationId) {
    // Local variables.
    $campaignArray = [];
    $campaignId = $campaign->id();
    $ids = '';
    if (is_numeric($campaignId)) {
      // Check if the entity exists.
      $ids = $this->entityTypeManager->getStorage('campaign')->getQuery('AND')
        ->condition('id', $campaignId, '=')
        ->execute();
    }
    else {
      drupal_set_message($this->t("Unable to find campaign."), 'error');
    }

    if ($ids) {
      // View builder.
      $entityType = 'campaign';
      $viewDisplay = 'campaign_kit';

      // Fields to render.
      $fields = [
        'title',
        'parent_id',
        'campaign_type',
        'header_image',
        'mail',
        'thank_you_message',
        'donation_page_title',
        'donation_page_description',
      ];

      // Get the Campaign entity object.
      /* @var $campaignEntity \Drupal\campaign_kit_core\Entity\Campaign */
      $campaignEntity = Campaign::load($campaignId);
      // Get the View for this Campaign entity object.
      $viewBuilder = $this->entityTypeManager->getViewBuilder($entityType);

      // Build the array to render.
      foreach ($fields as $field_name) {
        if ($campaignEntity->hasField($field_name) && $campaignEntity->access('view')) {
          $value = $campaignEntity->get($field_name);
          $output = $viewBuilder->viewField($value, $viewDisplay);
          $output['#cache']['tags'] = $campaignEntity->getCacheTags();
          $campaignArray[$field_name] = $output;
        }
      }
      // Check if it is a parent or child.
      if ($campaignEntity->get('campaign_type')->value == 'child') {
        /* @var $parent \Drupal\campaign_kit_core\Entity\Campaign */
        $parent = $campaignEntity->getParent();
        // Back to campaign link.
        $backToCampaignUrl = Url::fromRoute('campaign_kit.campaign_kit_controller_view_campaign_child_entity', [
          'campaignParentId' => $parent->id(),
          'campaign' => $campaignId,
        ]);
        $backToCampaignLink = Link::fromTextAndUrl($this->t('Back to Campaign Page!'), $backToCampaignUrl);
        $campaignArray['back_to_campaign_link'] = $backToCampaignLink->toRenderable();
        $campaignArray['back_to_campaign_link']['#attributes'] = [
          'class' => [
            'donate-button',
            'campaign-default--button',
          ],
        ];

      }
      else {
        // Back to campaign link.
        $backToCampaignUrl = Url::fromRoute('campaign_kit.campaign_kit_controller_view_campaign_entity', [
          'campaign' => $campaignId,
        ]);
        $backToCampaignLink = Link::fromTextAndUrl($this->t('Back to Campaign Page!'), $backToCampaignUrl);
        $campaignArray['back_to_campaign_link'] = $backToCampaignLink->toRenderable();
        $campaignArray['back_to_campaign_link']['#attributes'] = [
          'class' => [
            'donate-button',
            'campaign-default--button',
          ],
        ];
      }

      // Query Campaign Donation fields.
      $campaignDonationId = $this->entityTypeManager->getStorage('campaign_donation')->getQuery('AND')
        ->condition('id', $campaignDonationId, '=')
        ->condition('campaign_id', $campaignId, '=')
        ->execute();

      if ($campaignDonationId) {
        // Load Campaign Donation entity.
        /* @var $campaignDonationEntity \Drupal\campaign_kit_core\Entity\CampaignDonation */
        $campaignDonationEntity = CampaignDonation::load(reset($campaignDonationId));

        // View builder.
        $campaignDonationEntityType = 'campaign_donation';
        $viewModeCampaignDonation = 'default';

        // Fields to render.
        $fieldsCampaignDonation = [
          'payment_processor',
          'recipient_message',
        ];

        $viewBuilderCampaignDonation = $this->entityTypeManager->getViewBuilder($campaignDonationEntityType);

        foreach ($fieldsCampaignDonation as $field_name) {
          if ($campaignDonationEntity->hasField($field_name) && $campaignDonationEntity->access('view')) {
            $value2 = $campaignDonationEntity->get($field_name);
            $output2 = $viewBuilderCampaignDonation->viewField($value2, $viewModeCampaignDonation);
            $output2['#cache']['tags'] = $campaignDonationEntity->getCacheTags();
            $campaignArray[$field_name] = $output2;
          }
        }

        if ($campaignDonationEntity->get('dedicate_donation')->value == '1') {
          $campaignArray['dedicate_donation'] = TRUE;
        }
        else {
          $campaignArray['dedicate_donation'] = FALSE;
        }
        // Getting just values.
        $campaignArray['dedicate_type'] = $campaignDonationEntity->get('dedicate_type')->value;
        $campaignArray['honoree_first_name'] = $campaignDonationEntity->get('honoree_first_name')->value;
        $campaignArray['honoree_last_name'] = $campaignDonationEntity->get('honoree_last_name')->value;
        $campaignArray['recipient_mail'] = $campaignDonationEntity->get('recipient_mail')->value;
        $campaignArray['recipient_first_name'] = $campaignDonationEntity->get('recipient_first_name')->value;
        $campaignArray['recipient_last_name'] = $campaignDonationEntity->get('recipient_last_name')->value;

      }
      else {
        drupal_set_message($this->t("There are no donations for this campaign yet."), 'error');
      }

    }
    else {
      drupal_set_message($this->t("Unable to find campaign."), 'error');
    }
    $response = [
      '#theme' => 'campaign--thank-you',
      '#campaign' => $campaignArray,
    ];

    return $response;
  }

  /**
   * Thank you with Campaign ID page.
   *
   * @param int $campaignParentId
   *   The Campaign Parent ID.
   * @param int $campaignChildId
   *   The Campaign Child ID.
   * @param int $campaignDonationId
   *   The Campaign Donation ID.
   *
   * @return array
   *   Returns an array for Thank You page.
   */
  public function thankYouChild($campaignParentId, $campaignChildId, $campaignDonationId) {
    $campaign = [];
    $parentId = '';
    if (is_numeric($campaignParentId)) {
      // Check if the entity exists.
      $parentId = $this->entityTypeManager->getStorage('campaign')->getQuery('AND')
        ->condition('id', $campaignParentId, '=')
        ->execute();
    }
    else {
      drupal_set_message($this->t("Unable to find campaign."), 'error');
    }

    if ($parentId) {
      // Check if the Campaign child entity exists.
      $childId = $this->entityTypeManager->getStorage('campaign')->getQuery('AND')
        ->condition('id', $campaignChildId, '=')
        ->condition('campaign_type', 'child', '=')
        ->execute();

      if ($childId) {
        // View builder.
        $entityType = 'campaign';
        $viewDisplay = 'campaign_kit';

        // Fields to render.
        $fields = [
          'title',
          'parent_id',
          'campaign_type',
          'header_image',
          'confirmation_email',
          'thank_you_message',
          'donation_page_title',
          'donation_page_description',
        ];

        // Get the Campaign entity object.
        /* @var $campaignEntity \Drupal\campaign_kit_core\Entity\Campaign */
        $campaignEntity = Campaign::load($campaignChildId);
        // Get the View for this Campaign entity object.
        $viewBuilder = $this->entityTypeManager->getViewBuilder($entityType);

        // Build the array to render.
        foreach ($fields as $field_name) {
          if ($campaignEntity->hasField($field_name) && $campaignEntity->access('view')) {
            $value = $campaignEntity->get($field_name);
            $output = $viewBuilder->viewField($value, $viewDisplay);
            // $output['#cache']['tags'] = $campaignEntity->getCacheTags();
            $campaign[$field_name] = $output;
          }
        }

        // Check if it is a parent or child.
        if ($campaignEntity->get('campaign_type')->value == 'child') {
          // Back to campaign link.
          $backToCampaignUrl = Url::fromRoute('campaign_kit.campaign_kit_controller_view_campaign_entity', [
            'campaign' => $campaignChildId,
          ]);
          $backToCampaignLink = Link::fromTextAndUrl($this->t('Back to Campaign Page!'), $backToCampaignUrl);
          $campaign['back_to_campaign_link'] = $backToCampaignLink->toRenderable();
          $campaign['back_to_campaign_link']['#attributes'] = [
            'class' => [
              'donate-button',
              'campaign-default--button',
            ],
          ];
        }

        // Query Campaign Donation fields.
        $campaignDonationId = $this->entityTypeManager->getStorage('campaign_donation')->getQuery('AND')
          // ->condition('first_name', 'Andre CK', '=')
          ->condition('id', $campaignDonationId, '=')
          ->condition('campaign_id', $campaignChildId, '=')
          ->execute();

        if ($campaignDonationId) {
          // Load Campaign Donation entity.
          /* @var $campaignDonationEntity \Drupal\campaign_kit_core\Entity\CampaignDonation */
          $campaignDonationEntity = CampaignDonation::load(reset($campaignDonationId));

          // View builder.
          $campaignDonationEntityType = 'campaign_donation';
          $viewModeCampaignDonation = 'default';

          // Fields to render.
          $fieldsCampaignDonation = [
            'payment_processor',
            'recipient_message',
          ];

          $viewBuilderCampaignDonation = $this->entityTypeManager->getViewBuilder($campaignDonationEntityType);

          foreach ($fieldsCampaignDonation as $field_name) {
            if ($campaignDonationEntity->hasField($field_name) && $campaignDonationEntity->access('view')) {
              $value2 = $campaignDonationEntity->get($field_name);
              $output2 = $viewBuilderCampaignDonation->viewField($value2, $viewModeCampaignDonation);
              $output2['#cache']['tags'] = $campaignDonationEntity->getCacheTags();
              $campaign[$field_name] = $output2;
            }
          }

          // Set True or False for 'dedicate_donation' field.
          if ($campaignDonationEntity->get('dedicate_donation')->value == '1') {
            $campaign['dedicate_donation'] = TRUE;
          }
          else {
            $campaign['dedicate_donation'] = FALSE;
          }
          // TODO: Check recipient message.
          $campaign['dedicate_type'] = $campaignDonationEntity->get('dedicate_type')->value;
          $campaign['honoree_first_name'] = $campaignDonationEntity->get('honoree_first_name')->value;
          $campaign['honoree_last_name'] = $campaignDonationEntity->get('honoree_last_name')->value;
          $campaign['recipient_mail'] = $campaignDonationEntity->get('recipient_mail')->value;
          $campaign['recipient_first_name'] = $campaignDonationEntity->get('recipient_first_name')->value;
          $campaign['recipient_last_name'] = $campaignDonationEntity->get('recipient_last_name')->value;
        }
        else {
          drupal_set_message($this->t("There are no donations for this campaign yet."), 'error');
        }

      }
      else {
        drupal_set_message($this->t("Unable to find campaign."), 'error');
      }
    }
    else {
      drupal_set_message($this->t("Unable to find campaign."), 'error');
    }

    // Response.
    $response = [
      '#theme' => 'campaign--child--thank-you',
      '#campaignChild' => $campaign,
    ];

    return $response;
  }

  /**
   * Returns Campaign title.
   *
   * @param \Drupal\Core\Entity\EntityInterface $campaign
   *   The Campaign entity.
   *
   * @return string
   *   Return the Campaign title.
   */
  public function getCampaignTitle(EntityInterface $campaign) {
    $campaignTitle = '';
    if ($campaign instanceof Campaign) {
      $campaignId = $campaign->id();
      // Check if the entity exists.
      $ids = $this->entityTypeManager->getStorage('campaign')->getQuery('AND')
        ->condition('id', $campaignId, '=')
        ->execute();

      if ($ids) {
        /* @var $campaignEntity \Drupal\campaign_kit_core\Entity\Campaign */
        $campaignEntity = Campaign::load($campaignId);
        $campaignTitle = $campaignEntity->get('title')->value;
      }
    }
    return $campaignTitle;
  }

  /**
   * Get Leaderboard table by Campaign.
   *
   * @param \Drupal\Core\Entity\EntityInterface $campaign
   *   The Campaign entity.
   *
   * @return mixed
   *   Return a table.
   */
  public function getLeaderboardByCampaign(EntityInterface $campaign = NULL) {
    $rows = [];
    $headers = [];
    /* @var $campaign \Drupal\campaign_kit_core\Entity\Campaign */
    if ($campaign instanceof Campaign) {
      $campaign_id = $campaign->id();
      $leaderBoard = $this->t('Leaderboard');
      $response['#markup'] = $campaign->get('title')->value . ' ' . $leaderBoard;
      // TODO: Add service to Controller.
      $connection = $this->connection;
      $query = NULL;
      $query = $connection->select('team_field_data', 'tfd')->fields('tfd', ['id', 'name']);
      $query->addExpression("(SELECT max(created) FROM campaign_donation_field_data cdfd1 WHERE cdfd1.campaign_id = :campaign_id AND cdfd1.team_id=tfd.id)", 'lastDonationDate', [':campaign_id' => $campaign_id]);
      $query->addExpression("(SELECT count(name) FROM campaign_donation_field_data cdfd6 WHERE cdfd6.campaign_id = :campaign_id AND cdfd6.team_id=tfd.id)", 'totalDonors', [':campaign_id' => $campaign_id]);
      switch ($campaign->getFrequencyAllowed()) {
        case 'onetime':
          $headers = [
            $this->t('Team Name'),
            $this->t('# of members'),
            $this->t('Last Donation'),
            $this->t('One-Time Total'),
            $this->t('One-Time Donors'),
          ];
          $query->addExpression("(SELECT sum(amount) FROM campaign_donation_field_data cdfd2 WHERE cdfd2.campaign_id = :campaign_id AND cdfd2.team_id=tfd.id AND cdfd2.donation_frequency = 'onetime')", 'countOneTime', [':campaign_id' => $campaign_id]);
          $query->addExpression("(SELECT count(*) FROM campaign_donation_field_data cdfd3 WHERE cdfd3.campaign_id = :campaign_id AND cdfd3.team_id=tfd.id AND cdfd3.donation_frequency = 'onetime')", 'totalOneTime', [':campaign_id' => $campaign_id]);
          break;

        case 'monthly':
          $headers = [
            $this->t('Team Name'),
            $this->t('# of members'),
            $this->t('Last Donation'),
            $this->t('Monthly Total'),
            $this->t('Monthly Donors'),
          ];
          $query->addExpression("(SELECT sum(amount) FROM campaign_donation_field_data cdfd4 WHERE cdfd4.campaign_id = :campaign_id AND cdfd4.team_id=tfd.id AND cdfd4.donation_frequency = 'monthly')", 'countMonthly', [':campaign_id' => $campaign_id]);
          $query->addExpression("(SELECT count(*) FROM campaign_donation_field_data cdfd5 WHERE cdfd5.campaign_id = :campaign_id AND cdfd5.team_id=tfd.id AND cdfd5.donation_frequency = 'monthly')", 'totalMonthly', [':campaign_id' => $campaign_id]);
          break;

        case 'both':
          $headers = [
            $this->t('Team Name'),
            $this->t('# of members'),
            $this->t('Last Donation'),
            $this->t('One-Time Total'),
            $this->t('One-Time Donors'),
            $this->t('Monthly Total'),
            $this->t('Monthly Donors'),
          ];
          $query->addExpression("(SELECT sum(amount) FROM campaign_donation_field_data cdfd2 WHERE cdfd2.campaign_id = :campaign_id AND cdfd2.team_id=tfd.id AND cdfd2.donation_frequency = 'onetime')", 'countOneTime', [':campaign_id' => $campaign_id]);
          $query->addExpression("(SELECT count(*) FROM campaign_donation_field_data cdfd3 WHERE cdfd3.campaign_id = :campaign_id AND cdfd3.team_id=tfd.id AND cdfd3.donation_frequency = 'onetime')", 'totalOneTime', [':campaign_id' => $campaign_id]);
          $query->addExpression("(SELECT sum(amount) FROM campaign_donation_field_data cdfd4 WHERE cdfd4.campaign_id = :campaign_id AND cdfd4.team_id=tfd.id AND cdfd4.donation_frequency = 'monthly')", 'countMonthly', [':campaign_id' => $campaign_id]);
          $query->addExpression("(SELECT count(*) FROM campaign_donation_field_data cdfd5 WHERE cdfd5.campaign_id = :campaign_id AND cdfd5.team_id=tfd.id AND cdfd5.donation_frequency = 'monthly')", 'totalMonthly', [':campaign_id' => $campaign_id]);
          break;
      }

      $query->condition('campaign_id', $campaign_id, '=');
      // Order by name.
      $query->orderBy('name');

      $result = $query->execute();
      $records = $result->fetchAll();

      foreach ($records as $entry => $value) {
        /* @var $datetime \Drupal\Core\Datetime\DrupalDateTime */
        if ($value->lastDonationDate) {
          $lastDonationDate = $this->dateFormatter->format($value->lastDonationDate, 'short');
        }
        else {
          $lastDonationDate = '';
        }
        // Get currency from plugin.
        $config = $this->configFactory->get('campaign_kit.settings');
        $clientPlugin = $config->get('campaign_kit');
        $legendCurrency = $this->getCurrency($clientPlugin['currency']);
        $teamName = Link::createFromRoute(
          Team::load($value->id)->getName(),
          'entity.team.canonical',
          ['team' => $value->id]
        );

        switch ($campaign->getFrequencyAllowed()) {
          case 'onetime':
            $rows[] = [
              'id_team' => $teamName,
              'number_members' => $value->totalDonors,
              'last_donation_date' => $lastDonationDate,
              'count_one_time' => $value->countOneTime ? $legendCurrency . $value->countOneTime : '',
              'total_one_time' => $value->totalOneTime ? $value->totalOneTime : '',
            ];
            break;

          case 'monthly':
            $rows[] = [
              'id_team' => $teamName,
              'number_members' => $value->totalDonors,
              'last_donation_date' => $lastDonationDate,
              'count_monthly' => $value->countMonthly ? $legendCurrency . $value->countMonthly : '',
              'total_monthly' => $value->totalMonthly ? $value->totalMonthly : '',
            ];
            break;

          case 'both':
            $rows[] = [
              'id_team' => $teamName,
              'number_members' => $value->totalDonors,
              'last_donation_date' => $lastDonationDate,
              'count_one_time' => $value->countOneTime ? $legendCurrency . $value->countOneTime : '',
              'total_one_time' => $value->totalOneTime ? $value->totalOneTime : '',
              'count_monthly' => $value->countMonthly ? $legendCurrency . $value->countMonthly : '',
              'total_monthly' => $value->totalMonthly ? $value->totalMonthly : '',
            ];
            break;
        }
      }
    }

    $response['campaign_kit_demo_table'] = [
      '#rows' => $rows,
      '#header' => $headers,
      '#type' => 'table',
      '#empty' => $this->t('This Campaign does not have any team yet.'),
    ];

    return $response;
  }

  /**
   * Gets the currency code.
   *
   * @param string $currency
   *   The Currency code to be validated.
   *
   * @return string
   *   The Currency Code.
   */
  private function getCurrency($currency) {
    $currencyCode = '';
    switch ($currency) {
      case 'USD':
        $currencyCode = '$';
        break;

    }
    return $currencyCode;
  }

}
