<?php

namespace Drupal\campaign_kit\Form;

use Drupal\campaign_kit\Campaign\CampaignPayment;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ConfigurationForm.
 */
class ConfigurationForm extends FormBase {

  /**
   * The Campaign Service.
   *
   * @var \Drupal\campaign_kit\Campaign\CampaignPayment
   */
  protected $campaignService;

  /**
   * ConfigurationForm constructor.
   *
   * @param \Drupal\campaign_kit\Campaign\CampaignPayment $campaignService
   */
  public function __construct(CampaignPayment $campaignService) {
    $this->campaign_service = $campaignService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
    // Load the service required to construct this class.
      $container->get('campaign_kit.campaign_payment')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'payment_processor_form';
  }

  /**
   * Campaign Kit processor form.
   *
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // TODO: Load the configuration using the machine name
    // TODO: of the payment plugin.
    $campaignKitConfig = \Drupal::config('campaign_kit.settings');
    $configurationForm = $campaignKitConfig->getRawData();

    // Campaign Kit Notification email.
    $form['campaign_kit_notification_email'] = [
      '#type' => 'email',
      '#title' => $this->t('Notification Email'),
      '#default_value' => $configurationForm['campaign_kit']['notification_email'],
    ];

    // Currency.
    $form['currency'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Currency'),
      '#maxlength' => 6,
      '#size' => 6,
      '#default_value' => $configurationForm['campaign_kit']['currency'],
    ];

    // Allowed frequency.
    $form['frequency'] = [
      '#type' => 'select',
      '#title' => $this->t('Frequency allowed'),
      '#default_value' => ($configurationForm['campaign_kit']['frequency'] != 'none' ? $configurationForm['campaign_kit']['frequency'] : 'empty_option'),
      '#options' => [
        'onetime' => $this->t('One-time'),
        'monthly' => $this->t('Monthly'),
        'both' => $this->t('Both'),
      ],
      '#empty_option' => $this->t('- None -'),
    ];

    $form['debug_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Debug mode'),
      '#default_value' => $configurationForm['campaign_kit']['debug_mode'],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * Saves configuration.
   *
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // TODO: Save the configuration using the machine name
    // TODO: of the payment plugin.
    $config_factory = \Drupal::configFactory();
    $config = $config_factory->getEditable('campaign_kit.settings');
    $config->set('campaign_kit.notification_email', $form_state->getValue('campaign_kit_notification_email'));
    $config->set('campaign_kit.currency', $form_state->getValue('currency'));
    $config->set('campaign_kit.frequency', $form_state->getValue('frequency'));
    $config->set('campaign_kit.debug_mode', $form_state->getValue('debug_mode'));
    $config->save(TRUE);

    drupal_set_message(t('Configuration saved.'));
  }

}
